@@ -18,15 +18,17 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include <cstdarg>
 #include <cstdlib>
 #include <iomanip>
 #include <memory>
 #include <sstream>
 #include <string>
-#include <strings.h>
+#include <sys/time.h>
 
 #include "common.h"
 #include "s3fs_logger.h"
+#include "string_util.h"
 
 //-------------------------------------------------------------------
 // S3fsLog class : variables
@@ -35,7 +37,7 @@ constexpr char          S3fsLog::LOGFILEENV[];
 constexpr const char*   S3fsLog::nest_spaces[];
 constexpr char          S3fsLog::MSGTIMESTAMP[];
 S3fsLog*                S3fsLog::pSingleton       = nullptr;
-S3fsLog::s3fs_log_level S3fsLog::debug_level      = S3fsLog::LEVEL_CRIT;
+S3fsLog::Level          S3fsLog::debug_level      = S3fsLog::Level::CRIT;
 FILE*                   S3fsLog::logfp            = nullptr;
 std::string             S3fsLog::logfile;
 bool                    S3fsLog::time_stamp       = true;
@@ -43,9 +45,9 @@ bool                    S3fsLog::time_stamp       = true;
 //-------------------------------------------------------------------
 // S3fsLog class : class methods
 //-------------------------------------------------------------------
-bool S3fsLog::IsS3fsLogLevel(s3fs_log_level level)
+bool S3fsLog::IsS3fsLogLevel(S3fsLog::Level level)
 {
-    return (level == (S3fsLog::debug_level & level));
+    return static_cast<int>(level) == (static_cast<int>(S3fsLog::debug_level) & static_cast<int>(level));
 }
 
 std::string S3fsLog::GetCurrentTime()
@@ -95,7 +97,7 @@ bool S3fsLog::ReopenLogfile()
     return S3fsLog::pSingleton->LowSetLogfile(tmp.c_str());
 }
 
-S3fsLog::s3fs_log_level S3fsLog::SetLogLevel(s3fs_log_level level)
+S3fsLog::Level S3fsLog::SetLogLevel(S3fsLog::Level level)
 {
     if(!S3fsLog::pSingleton){
         S3FS_PRN_CRIT("S3fsLog::pSingleton is nullptr.");
@@ -104,7 +106,7 @@ S3fsLog::s3fs_log_level S3fsLog::SetLogLevel(s3fs_log_level level)
     return S3fsLog::pSingleton->LowSetLogLevel(level);
 }
 
-S3fsLog::s3fs_log_level S3fsLog::BumpupLogLevel()
+S3fsLog::Level S3fsLog::BumpupLogLevel()
 {
     if(!S3fsLog::pSingleton){
         S3FS_PRN_CRIT("S3fsLog::pSingleton is nullptr.");
@@ -146,7 +148,7 @@ S3fsLog::~S3fsLog()
         }
         S3fsLog::logfile.clear();
         S3fsLog::pSingleton  = nullptr;
-        S3fsLog::debug_level = S3fsLog::LEVEL_CRIT;
+        S3fsLog::debug_level = Level::CRIT;
 
         closelog();
     }else{
@@ -167,9 +169,10 @@ bool S3fsLog::LowLoadEnv()
         }
     }
     if(nullptr != (pEnvVal = getenv(S3fsLog::MSGTIMESTAMP))){
-        if(0 == strcasecmp(pEnvVal, "true") || 0 == strcasecmp(pEnvVal, "yes") || 0 == strcasecmp(pEnvVal, "1")){
+        auto env_val = CaseInsensitiveStringView(pEnvVal);
+        if(env_val == "true" || env_val == "yes" || env_val == "1"){
             S3fsLog::time_stamp = true;
-        }else if(0 == strcasecmp(pEnvVal, "false") || 0 == strcasecmp(pEnvVal, "no") || 0 == strcasecmp(pEnvVal, "0")){
+        }else if(env_val == "false" || env_val == "no" || env_val == "0"){
             S3fsLog::time_stamp = false;
         }else{
             S3FS_PRN_WARN("Unknown %s environment value(%s) is specified, skip to set time stamp mode.", S3fsLog::MSGTIMESTAMP, pEnvVal);
@@ -218,7 +221,7 @@ bool S3fsLog::LowSetLogfile(const char* pfile)
     return true;
 }
 
-S3fsLog::s3fs_log_level S3fsLog::LowSetLogLevel(s3fs_log_level level)
+S3fsLog::Level S3fsLog::LowSetLogLevel(Level level)
 {
     if(S3fsLog::pSingleton != this){
         S3FS_PRN_ERR("This object is not as same as S3fsLog::pSingleton.");
@@ -227,73 +230,69 @@ S3fsLog::s3fs_log_level S3fsLog::LowSetLogLevel(s3fs_log_level level)
     if(level == S3fsLog::debug_level){
         return S3fsLog::debug_level;
     }
-    s3fs_log_level old   = S3fsLog::debug_level;
+    Level old   = S3fsLog::debug_level;
     S3fsLog::debug_level = level;
     setlogmask(LOG_UPTO(GetSyslogLevel(S3fsLog::debug_level)));
     S3FS_PRN_CRIT("change debug level from %sto %s", GetLevelString(old), GetLevelString(S3fsLog::debug_level));
     return old;
 }
 
-S3fsLog::s3fs_log_level S3fsLog::LowBumpupLogLevel()
+S3fsLog::Level S3fsLog::LowBumpupLogLevel() const
 {
     if(S3fsLog::pSingleton != this){
         S3FS_PRN_ERR("This object is not as same as S3fsLog::pSingleton.");
         return S3fsLog::debug_level;    // Although it is an error, it returns the current value.
     }
-    s3fs_log_level old   = S3fsLog::debug_level;
-    S3fsLog::debug_level = ( LEVEL_CRIT == S3fsLog::debug_level ? LEVEL_ERR  :
-                             LEVEL_ERR  == S3fsLog::debug_level ? LEVEL_WARN :
-                             LEVEL_WARN == S3fsLog::debug_level ? LEVEL_INFO :
-                             LEVEL_INFO == S3fsLog::debug_level ? LEVEL_DBG  : LEVEL_CRIT );
+    Level old   = S3fsLog::debug_level;
+    S3fsLog::debug_level = ( Level::CRIT == S3fsLog::debug_level ? Level::ERR  :
+                             Level::ERR  == S3fsLog::debug_level ? Level::WARN :
+                             Level::WARN == S3fsLog::debug_level ? Level::INFO :
+                             Level::INFO == S3fsLog::debug_level ? Level::DBG  : Level::CRIT );
     setlogmask(LOG_UPTO(GetSyslogLevel(S3fsLog::debug_level)));
     S3FS_PRN_CRIT("change debug level from %sto %s", GetLevelString(old), GetLevelString(S3fsLog::debug_level));
     return old;
 }
 
-void s3fs_low_logprn(S3fsLog::s3fs_log_level level, const char* file, const char *func, int line, const char *fmt, ...)
+void s3fs_low_logprn(S3fsLog::Level level, const char* file, const char *func, int line, const char *fmt, ...)
 {
-    if(S3fsLog::IsS3fsLogLevel(level)){
-        va_list va;
-        va_start(va, fmt);
-        size_t len = vsnprintf(nullptr, 0, fmt, va) + 1;
-        va_end(va);
+    va_list va;
+    va_start(va, fmt);
+    size_t len = vsnprintf(nullptr, 0, fmt, va) + 1;
+    va_end(va);
 
-        std::unique_ptr<char[]> message(new char[len]);
-        va_start(va, fmt);
-        vsnprintf(message.get(), len, fmt, va);
-        va_end(va);
+    auto message = std::make_unique<char[]>(len);
+    va_start(va, fmt);
+    vsnprintf(message.get(), len, fmt, va);
+    va_end(va);
 
-        if(foreground || S3fsLog::IsSetLogFile()){
-            S3fsLog::SeekEnd();
-            fprintf(S3fsLog::GetOutputLogFile(), "%s%s%s:%s(%d): %s\n", S3fsLog::GetCurrentTime().c_str(), S3fsLog::GetLevelString(level), file, func, line, message.get());
-            S3fsLog::Flush();
-        }else{
-            // TODO: why does this differ from s3fs_low_logprn2?
-            syslog(S3fsLog::GetSyslogLevel(level), "%s%s:%s(%d): %s", instance_name.c_str(), file, func, line, message.get());
-        }
+    if(foreground || S3fsLog::IsSetLogFile()){
+        S3fsLog::SeekEnd();
+        fprintf(S3fsLog::GetOutputLogFile(), "%s%s%s:%s(%d): %s\n", S3fsLog::GetCurrentTime().c_str(), S3fsLog::GetLevelString(level), file, func, line, message.get());
+        S3fsLog::Flush();
+    }else{
+        // TODO: why does this differ from s3fs_low_logprn2?
+        syslog(S3fsLog::GetSyslogLevel(level), "%s%s:%s(%d): %s", instance_name.c_str(), file, func, line, message.get());
     }
 }
 
-void s3fs_low_logprn2(S3fsLog::s3fs_log_level level, int nest, const char* file, const char *func, int line, const char *fmt, ...)
+void s3fs_low_logprn2(S3fsLog::Level level, int nest, const char* file, const char *func, int line, const char *fmt, ...)
 {
-    if(S3fsLog::IsS3fsLogLevel(level)){
-        va_list va;
-        va_start(va, fmt);
-        size_t len = vsnprintf(nullptr, 0, fmt, va) + 1;
-        va_end(va);
+    va_list va;
+    va_start(va, fmt);
+    size_t len = vsnprintf(nullptr, 0, fmt, va) + 1;
+    va_end(va);
 
-        std::unique_ptr<char[]> message(new char[len]);
-        va_start(va, fmt);
-        vsnprintf(message.get(), len, fmt, va);
-        va_end(va);
+    auto message = std::make_unique<char[]>(len);
+    va_start(va, fmt);
+    vsnprintf(message.get(), len, fmt, va);
+    va_end(va);
 
-        if(foreground || S3fsLog::IsSetLogFile()){
-            S3fsLog::SeekEnd();
-            fprintf(S3fsLog::GetOutputLogFile(), "%s%s%s%s:%s(%d): %s\n", S3fsLog::GetCurrentTime().c_str(), S3fsLog::GetLevelString(level), S3fsLog::GetS3fsLogNest(nest), file, func, line, message.get());
-            S3fsLog::Flush();
-        }else{
-            syslog(S3fsLog::GetSyslogLevel(level), "%s%s%s", instance_name.c_str(), S3fsLog::GetS3fsLogNest(nest), message.get());
-        }
+    if(foreground || S3fsLog::IsSetLogFile()){
+        S3fsLog::SeekEnd();
+        fprintf(S3fsLog::GetOutputLogFile(), "%s%s%s%s:%s(%d): %s\n", S3fsLog::GetCurrentTime().c_str(), S3fsLog::GetLevelString(level), S3fsLog::GetS3fsLogNest(nest), file, func, line, message.get());
+        S3fsLog::Flush();
+    }else{
+        syslog(S3fsLog::GetSyslogLevel(level), "%s%s%s", instance_name.c_str(), S3fsLog::GetS3fsLogNest(nest), message.get());
     }
 }
 
